/**************************************************************************
 *                                                                        *
 *   Copyright (c) Digital Equipment  Corporation,  1991.   All  Rights   *
 *   Reserved.  Unpublished rights reserved under the copyright laws of   *
 *   the United States.                                                   *
 *                                                                        *
 *   The software  contained  on  this  media  is  proprietary  to  and   *
 *   embodies   the   confidential   technology  of  Digital  Equipment   *
 *   Corporation.  Possession, use, duplication or dissemination of the   *
 *   software  and media is authorized only pursuant to a valid written   *
 *   license from Digital Equipment Corporation.                          *
 *                                                                        *
 *   RESTRICTED RIGHTS LEGEND Use, duplication, or  disclosure  by  the   *
 *   U.S.   Government  is  subject  to  restrictions  as  set forth in   *
 *   Subparagraph  (c)(1)(ii)  of  DFARS  252.227-7013,   or   in   FAR   *
 *   52.227-19, as applicable.                                            *
 *                                                                        *
 **************************************************************************/
#ifdef SCCS
static char sccsid[]="@(#)read_plot3d.c	6.1 Stardent 91/08/26";
#endif
/*
			Copyright (c) 1989 by
			Stardent Computer Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Stardent Computer Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under sccs control at Stardent in:
	@phobos/sccs3.p/avs/examples/s.read_plot3d.c
	
*/
#include <stdio.h>
 
#include <avs/vex.h>
#include <avs/flow.h>
#include <avs/geom.h>
#include <avs/field.h>
#include <avs/geomdata.h>
#include <avs/colormap.h>
 
extern int FlowVerbose;
 
#define GAMMA 1.4
 
/******************************************************************************/
/*
 * Read a plot3D data set
 */
 
read_plot3d()
{
   int read_plot3d_compute();
   
   AVSset_module_name("read PLOT3D c", MODULE_DATA);
   AVScreate_output_port("Field Output", "field 3D scalar irregular 3-space float");
 
   AVSconnect_widget(AVSadd_parameter("Read Plot3D Data Browser (q)", "string", 0, 0, 0),  
		     "browser");
   AVSconnect_widget(AVSadd_parameter("Read Plot3D XYZ Browser (x)", "string", 0, 0, 0), 
		     "browser");
   AVSadd_parameter("Parameter", "choice", "temperature", 
		    "density x-momentum y-momentum z-momentum stagnation temperature",
		    " ");
   AVSset_compute_proc(read_plot3d_compute);
}
	
read_plot3d_compute(data, DataFileName, XYZFileName, parameter)
AVSfield_float **data;
char *DataFileName, *XYZFileName, *parameter;
{
    AVSfield_float *input;
    FILE 	*fp, *fxyz;
    int 	dims[3];
    int 	size, result = 1;
    int 	i, junk[4];
    float 	*density, *xmomentum, *ymomentum, *zmomentum, *stagnation;
    float 	rr, u, v, w, e, v2, am2, p, t;
 
    /* Make sure to free the field that was allocated last time */
    if (*data) AVSdata_free("field",*data);
 
    if (!DataFileName || !XYZFileName)
       return(0);
 
    /* Free old memory */
    if (*data) {
      if (FlowVerbose)
	 fprintf(stderr,"FREEING old data...\n");
      free(*data);
    }
 
    if ((fp=fopen(DataFileName, "r")) == NULL) {
      AVSerror("read_plot3d: cannot open %s\n", DataFileName);
      return(0);
    }
 
    if ((fxyz=fopen(XYZFileName, "r")) == NULL) {
      AVSerror("read_plot3d: cannot open %s\n", XYZFileName);
      return(0);
    }
 
    /* read q file header */
    fread(&dims[0], sizeof(int), 1, fp);
    fread(&dims[1], sizeof(int), 1, fp);
    fread(&dims[2], sizeof(int), 1, fp);
    size = dims[0] * dims[1] * dims[2];
    fprintf(stderr, "volume is %d x %d x %d\n", dims[0], dims[1], dims[2]);
    fread(junk, sizeof(int), 4, fp);
    
    /* Allocate space for volume data */
    input = (AVSfield_float *)AVSdata_alloc("field 3D scalar irregular 3-space float", dims);
    
    /* read xyz file header*/
    fread(junk, sizeof(int), 3, fxyz);
    if (junk[0] != dims[0] || junk[1] != dims[1] || junk[2] != dims[2]) {
      AVSerror("read_plot3d: mismatch: %d %d %d\n", junk[0], junk[1], junk[2]);
      return(0);
    }
 
    fprintf(stderr, "about to allocate memory...\n");
    
    density = (float *)ALLOC_LOCAL(size*sizeof(float));
    xmomentum = (float *)ALLOC_LOCAL(size*sizeof(float));
    ymomentum = (float *)ALLOC_LOCAL(size*sizeof(float));
    zmomentum = (float *)ALLOC_LOCAL(size*sizeof(float));
    stagnation = (float *)ALLOC_LOCAL(size*sizeof(float));
    
    fprintf(stderr, "reading XYZ...\n");
    fread(&(input->points[0]), sizeof(float), size, fxyz);
    fread(&(input->points[size]), sizeof(float), size, fxyz);
    fread(&(input->points[2*size]), sizeof(float), size, fxyz);
    
    fprintf(stderr, "reading density...\n");
    fread(density, sizeof(float), size, fp);
    fprintf(stderr, "reading xmomentum...\n");
    fread(xmomentum, sizeof(float), size, fp);
    fprintf(stderr, "reading ymomentum...\n");
    fread(ymomentum, sizeof(float), size, fp);
    fprintf(stderr, "reading zmomentum...\n");
    fread(zmomentum, sizeof(float), size, fp);
    fprintf(stderr, "reading stagnation...\n");
    fread(stagnation, sizeof(float), size, fp);
 
    if (!strcmp(parameter, "density")) {
      for(i=0; i<size; i++) 
	input->data[i] = density[i];
    } else if (!strcmp(parameter, "x-momentum")) {
      for(i=0; i<size; i++) 
	input->data[i] = xmomentum[i];
    } else if (!strcmp(parameter, "y-momentum")) {
      for(i=0; i<size; i++) 
	input->data[i] = ymomentum[i];
    } else if (!strcmp(parameter, "z-momentum")) {
      for(i=0; i<size; i++) 
	input->data[i] = zmomentum[i];
    } else if (!strcmp(parameter, "stagnation")) {
      for(i=0; i<size; i++) 
	input->data[i] = stagnation[i];
    } else if (!strcmp(parameter, "temperature")) {
      for(i=0; i<size; i++) {
	rr = 1.0 / density[i];
	u = xmomentum[i] * rr;
	v = ymomentum[i] * rr;
	w = zmomentum[i] * rr;
	e = stagnation[i] * rr;
	v2 = u*u + v*v + w*w;
	am2 = v2 / (GAMMA*(GAMMA-1.0)*(e - 0.5*v2));
	p = (GAMMA-1.0) * (stagnation[i]- 5.0*density[i]*v2);
	t = p*rr;
	input->data[i] = t*(1.0 + 0.5*(GAMMA-1.0)*am2);
      }
    } else {
       AVSerror("read plot3d: unknown choice: %s", parameter);
       result = 0;
    }
    
    FREE_LOCAL(density);
    FREE_LOCAL(xmomentum);
    FREE_LOCAL(ymomentum);
    FREE_LOCAL(zmomentum);
    FREE_LOCAL(stagnation);
 
    *data = input;
    
    fclose(fp);
    fclose(fxyz);
 
    return(result);
}
 
/*
 * initialize the modules contained in this file.
 */
 
int ((*data_mod_list[])()) = {
	read_plot3d,
};
 
#define NMODS sizeof(data_mod_list) / sizeof(char *) 
 
AVSinit_modules()
{
	AVSinit_from_module_list(data_mod_list, NMODS);
}
 
